@@ -36,6 +36,10 @@ build_c_helpers_dict(void)
 
 #define declpointer(ptr) _declpointer(#ptr, &numba_##ptr)
 
+    declmethod(fixed_fmod);
+    declmethod(fixed_fmodf);
+    declmethod(set_fnclex);
+
     declmethod(sdiv);
     declmethod(srem);
     declmethod(udiv);
@@ -94,6 +98,7 @@ build_c_helpers_dict(void)
     declmethod(xxpotrf);
     declmethod(ez_rgeev);
     declmethod(ez_cgeev);
+    declmethod(ez_xxxevd);
     declmethod(ez_gesdd);
     declmethod(ez_geqrf);
     declmethod(ez_xxgqr);
