@@ -53,21 +53,19 @@ class TypeAnnotation(object):
     # to be compiled).
     func_data = OrderedDict()
 
-    def __init__(self, interp, typemap, calltypes, lifted, lifted_from, args, return_type,
-                 func_attr, html_output=None):
-        self.filename = interp.bytecode.filename
-        self.func = interp.bytecode.func
-        self.blocks = interp.blocks
+    def __init__(self, func_ir, typemap, calltypes, lifted, lifted_from,
+                 args, return_type, html_output=None):
+        self.func_id = func_ir.func_id
+        self.blocks = func_ir.blocks
         self.typemap = typemap
         self.calltypes = calltypes
         if html_output is None:
             self.html_output = None
         else:
             self.html_output = os.path.join(os.getcwd(), html_output)
-        self.filename = interp.loc.filename
-        self.linenum = str(interp.loc.line)
+        self.filename = func_ir.loc.filename
+        self.linenum = str(func_ir.loc.line)
         self.signature = str(args) + ' -> ' + str(return_type)
-        self.func_attr = func_attr
 
         # lifted loop information
         self.lifted = lifted
@@ -114,7 +112,7 @@ class TypeAnnotation(object):
         return groupedinst
 
     def annotate(self):
-        source = SourceLines(self.func)
+        source = SourceLines(self.func_id.func)
         # if not source.avail:
         #     return "Source code unavailable"
 
@@ -153,7 +151,7 @@ class TypeAnnotation(object):
             return io.getvalue()
 
     def html_annotate(self, outfile=None):
-        python_source = SourceLines(self.func)
+        python_source = SourceLines(self.func_id.func)
         ir_lines = self.prepare_annotations()
         line_nums = [num for num in python_source]
         lifted_lines = [l.bytecode.firstlineno for l in self.lifted]
@@ -168,7 +166,7 @@ class TypeAnnotation(object):
             indent_len = len(_getindent(line))
             func_data['ir_indent'][num].append('&nbsp;' * indent_len)
 
-        func_key = (self.func_attr.filename + ':' + str(self.func_attr.lineno + 1),
+        func_key = (self.func_id.filename + ':' + str(self.func_id.firstlineno + 1),
                     self.signature)
         if self.lifted_from is not None and self.lifted_from[1]['num_lifted_loops'] > 0:
             # This is a lifted loop function that is being compiled. Get the
@@ -207,7 +205,7 @@ class TypeAnnotation(object):
             func_data['num_lifted_loops'] = self.num_lifted_loops
 
             func_data['filename'] = self.filename
-            func_data['funcname'] = self.func_attr.name
+            func_data['funcname'] = self.func_id.func_name
             func_data['python_lines'] = []
             func_data['python_indent'] = {}
             func_data['python_tags'] = {}
