@@ -9,9 +9,9 @@ import warnings
 import traceback
 from .tracing import trace, event
 
-from numba import (bytecode, interpreter, funcdesc, typing, typeinfer,
-                   lowering, objmode, utils, config, errors,
-                   types, ir, macro, types, rewrites, transforms)
+from numba import (bytecode, interpreter, funcdesc, postproc,
+                   typing, typeinfer, lowering, objmode, utils, config,
+                   errors, types, ir, types, rewrites, transforms)
 from numba.targets import cpu, callconv
 from numba.annotations import type_annotations
 
@@ -101,11 +101,6 @@ class CompileResult(namedtuple("_CompileResult", CR_FIELDS)):
         return cr
 
 
-FunctionAttributes = namedtuple("FunctionAttributes",
-                                ['name', 'filename', 'lineno'])
-DEFAULT_FUNCTION_ATTRIBUTES = FunctionAttributes('<anonymous>', '<unknown>', 0)
-
-
 _LowerResult = namedtuple("_LowerResult", [
     "fndesc",
     "call_helper",
@@ -115,38 +110,6 @@ _LowerResult = namedtuple("_LowerResult", [
 ])
 
 
-def get_function_attributes(func):
-    '''
-    Extract the function attributes from a Python function or object with
-    *py_func* attribute, such as CPUDispatcher.
-
-    Returns an instance of FunctionAttributes.
-    '''
-    if hasattr(func, 'py_func'):
-        func = func.py_func  # This is a Overload object
-
-    name, filename, lineno = DEFAULT_FUNCTION_ATTRIBUTES
-    try:
-        name = func.__name__
-    except AttributeError:
-        pass  # this "function" object isn't really a function
-
-    try:
-        possible_filename = inspect.getsourcefile(func)
-        # Sometimes getsourcefile returns null
-        if possible_filename is not None:
-            filename = possible_filename
-    except TypeError:
-        pass  # built-in function, or other object unsupported by inspect
-
-    try:
-        lines, lineno = inspect.getsourcelines(func)
-    except (IOError, TypeError):
-        pass  # unable to read source code for function
-
-    return FunctionAttributes(name, filename, lineno)
-
-
 def compile_result(**kws):
     keys = set(kws.keys())
     fieldset = set(CR_FIELDS)
@@ -179,6 +142,21 @@ def compile_isolated(func, args, return_type=None, flags=DEFAULT_FLAGS,
                              flags, locals)
 
 
+def run_frontend(func):
+    """
+    Run the compiler frontend over the given Python function, and return
+    the function's canonical Numba IR.
+    """
+    # XXX make this a dedicated Pipeline?
+    func_id = bytecode.FunctionIdentity.from_function(func)
+    interp = interpreter.Interpreter(func_id)
+    bc = bytecode.ByteCode(func_id=func_id)
+    func_ir = interp.interpret(bc)
+    post_proc = postproc.PostProcessor(func_ir)
+    post_proc.run()
+    return func_ir
+
+
 class _CompileStatus(object):
     """
     Used like a C record
@@ -286,10 +264,9 @@ class Pipeline(object):
         self.locals = locals
 
         # Results of various steps of the compilation pipeline
-        self.interp = None
         self.bc = None
-        self.func = None
-        self.func_attr = None
+        self.func_id = None
+        self.func_ir = None
         self.lifted = None
         self.lifted_from = None
         self.typemap = None
@@ -317,8 +294,8 @@ class Pipeline(object):
                     e = e.with_traceback(None)
                 warnings.warn_explicit('%s: %s' % (msg, e),
                                        errors.NumbaWarning,
-                                       self.func_attr.filename,
-                                       self.func_attr.lineno)
+                                       self.func_id.filename,
+                                       self.func_id.firstlineno)
 
                 raise
 
@@ -338,27 +315,26 @@ class Pipeline(object):
                     e = e.with_traceback(None)
                 warnings.warn_explicit('%s: %s' % (msg, e),
                                        errors.NumbaWarning,
-                                       self.func_attr.filename,
-                                       self.func_attr.lineno)
+                                       self.func_id.filename,
+                                       self.func_id.firstlineno)
 
                 raise
 
-    def extract_bytecode(self, func):
+    def extract_bytecode(self, func_id):
         """
         Extract bytecode from function
         """
-        func_attr = get_function_attributes(func)
-        self.func = func
-        self.func_attr = func_attr
-        bc = bytecode.ByteCode(func=self.func)
+        bc = bytecode.ByteCode(func_id)
         if config.DUMP_BYTECODE:
             print(bc.dump())
 
         return bc
 
     def compile_extra(self, func):
+        self.func_id = bytecode.FunctionIdentity.from_function(func)
+
         try:
-            bc = self.extract_bytecode(func)
+            bc = self.extract_bytecode(self.func_id)
         except BaseException as e:
             if self.status.can_giveup:
                 self.stage_compile_interp_mode()
@@ -366,47 +342,29 @@ class Pipeline(object):
             else:
                 raise e
 
-        return self.compile_bytecode(bc, func_attr=self.func_attr)
-
-    def compile_bytecode(self, bc, lifted=(), lifted_from=None,
-                         func_attr=DEFAULT_FUNCTION_ATTRIBUTES):
-        self.bc = bc
-        self.func = bc.func
-        self.lifted = lifted
-        self.lifted_from = lifted_from
-        self.func_attr = func_attr
-        return self._compile_bytecode()
-
-    def compile_ir(self, interp, lifted=(), lifted_from=None,
-                   func_attr=DEFAULT_FUNCTION_ATTRIBUTES):
-        self.bc = interp.bytecode
-        self.func = interp.bytecode.func
-        self.lifted = lifted
-        self.lifted_from = lifted_from
-        self.func_attr = func_attr
-
-        self._set_and_check_interp(interp)
-        return self._compile_ir()
-
-    def compile_internal(self, bc, func_attr=DEFAULT_FUNCTION_ATTRIBUTES):
-        assert not self.flags.force_pyobject
         self.bc = bc
         self.lifted = ()
-        self.func_attr = func_attr
-        self.status.can_fallback = False
-        self.status.can_giveup = False
+        self.lifted_from = None
         return self._compile_bytecode()
 
+    def compile_ir(self, func_ir, lifted=(), lifted_from=None):
+        self.func_id = func_ir.func_id
+        self.lifted = lifted
+        self.lifted_from = lifted_from
+
+        self._set_and_check_ir(func_ir)
+        return self._compile_ir()
+
     def stage_analyze_bytecode(self):
         """
         Analyze bytecode and translating to Numba IR
         """
-        interp = translate_stage(self.bc)
-        self._set_and_check_interp(interp)
+        func_ir = translate_stage(self.func_id, self.bc)
+        self._set_and_check_ir(func_ir)
 
-    def _set_and_check_interp(self, interp):
-        self.interp = interp
-        self.nargs = self.interp.arg_count
+    def _set_and_check_ir(self, func_ir):
+        self.func_ir = func_ir
+        self.nargs = self.func_ir.arg_count
         if not self.args and self.flags.force_pyobject:
             # Allow an empty argument types specification when object mode
             # is explicitly requested.
@@ -416,6 +374,9 @@ class Pipeline(object):
                             "but function takes %d arguments"
                             % (len(self.args), self.nargs))
 
+    def stage_process_ir(self):
+        ir_processing_stage(self.func_ir)
+
     def frontend_looplift(self):
         """
         Loop lifting analysis and transformation
@@ -428,7 +389,7 @@ class Pipeline(object):
         if not self.flags.enable_pyobject_looplift:
             loop_flags.unset('enable_pyobject')
 
-        main, loops = transforms.loop_lifting(self.interp,
+        main, loops = transforms.loop_lifting(self.func_ir,
                                               typingctx=self.typingctx,
                                               targetctx=self.targetctx,
                                               locals=self.locals,
@@ -442,8 +403,7 @@ class Pipeline(object):
             cres = compile_ir(self.typingctx, self.targetctx, main,
                               self.args, self.return_type,
                               outer_flags, self.locals,
-                              lifted=tuple(loops), lifted_from=None,
-                              func_attr=self.func_attr)
+                              lifted=tuple(loops), lifted_from=None)
             return cres
 
     def stage_objectmode_frontend(self):
@@ -466,18 +426,18 @@ class Pipeline(object):
         Type inference and legalization
         """
         with self.fallback_context('Function "%s" failed type inference'
-                                   % (self.func_attr.name,)):
+                                   % (self.func_id.func_name,)):
             # Type inference
             self.typemap, self.return_type, self.calltypes = type_inference_stage(
                 self.typingctx,
-                self.interp,
+                self.func_ir,
                 self.args,
                 self.return_type,
                 self.locals)
 
         with self.fallback_context('Function "%s" has invalid return type'
-                                   % (self.func_attr.name,)):
-            legalize_return_type(self.return_type, self.interp,
+                                   % (self.func_id.func_name,)):
+            legalize_return_type(self.return_type, self.func_ir,
                                  self.targetctx)
 
     def stage_generic_rewrites(self):
@@ -485,41 +445,40 @@ class Pipeline(object):
         Perform any intermediate representation rewrites before type
         inference.
         """
-        assert self.interp
+        assert self.func_ir
         with self.fallback_context('Internal error in pre-inference rewriting '
                                    'pass encountered during compilation of '
-                                   'function "%s"' % (self.func_attr.name,)):
+                                   'function "%s"' % (self.func_id.func_name,)):
             rewrites.rewrite_registry.apply('before-inference',
-                                            self, self.interp)
+                                            self, self.func_ir)
 
     def stage_nopython_rewrites(self):
         """
         Perform any intermediate representation rewrites after type
         inference.
         """
-        # Ensure we have an IR container (interp), and type information.
-        assert self.interp
+        # Ensure we have an IR and type information.
+        assert self.func_ir
         assert isinstance(getattr(self, 'typemap', None), dict)
         assert isinstance(getattr(self, 'calltypes', None), dict)
         with self.fallback_context('Internal error in post-inference rewriting '
                                    'pass encountered during compilation of '
-                                   'function "%s"' % (self.func_attr.name,)):
+                                   'function "%s"' % (self.func_id.func_name,)):
             rewrites.rewrite_registry.apply('after-inference',
-                                            self, self.interp)
+                                            self, self.func_ir)
 
     def stage_annotate_type(self):
         """
         Create type annotation after type inference
         """
         self.type_annotation = type_annotations.TypeAnnotation(
-            interp=self.interp,
+            func_ir=self.func_ir,
             typemap=self.typemap,
             calltypes=self.calltypes,
             lifted=self.lifted,
             lifted_from=self.lifted_from,
             args=self.args,
             return_type=self.return_type,
-            func_attr=self.func_attr,
             html_output=config.HTML)
 
         if config.ANNOTATE:
@@ -534,7 +493,7 @@ class Pipeline(object):
         Object mode compilation
         """
         with self.giveup_context("Function %s failed at object mode lowering"
-                                 % (self.func_attr.name,)):
+                                 % (self.func_id.func_name,)):
             if len(self.args) != self.nargs:
                 # append missing
                 self.args = (tuple(self.args) + (types.pyobject,) *
@@ -542,17 +501,17 @@ class Pipeline(object):
 
             return py_lowering_stage(self.targetctx,
                                      self.library,
-                                     self.interp,
+                                     self.func_ir,
                                      self.flags)
 
     def backend_nopython_mode(self):
         """Native mode compilation"""
         with self.fallback_context("Function %s failed at nopython "
-                                   "mode lowering" % (self.func_attr.name,)):
+                                   "mode lowering" % (self.func_id.func_name,)):
             return native_lowering_stage(
                 self.targetctx,
                 self.library,
-                self.interp,
+                self.func_ir,
                 self.typemap,
                 self.return_type,
                 self.calltypes,
@@ -564,7 +523,7 @@ class Pipeline(object):
         """
         if self.library is None:
             codegen = self.targetctx.codegen()
-            self.library = codegen.create_library(self.bc.func_qualname)
+            self.library = codegen.create_library(self.func_id.func_qualname)
             # Enable object caching upfront, so that the library can
             # be later serialized.
             self.library.enable_object_caching()
@@ -597,17 +556,17 @@ class Pipeline(object):
         # Warn if compiled function in object mode and force_pyobject not set
         if not self.flags.force_pyobject:
             if len(self.lifted) > 0:
-                warn_msg = 'Function "%s" was compiled in object mode without forceobj=True, but has lifted loops.' % (self.func_attr.name,)
+                warn_msg = 'Function "%s" was compiled in object mode without forceobj=True, but has lifted loops.' % (self.func_id.func_name,)
             else:
-                warn_msg = 'Function "%s" was compiled in object mode without forceobj=True.' % (self.func_attr.name,)
+                warn_msg = 'Function "%s" was compiled in object mode without forceobj=True.' % (self.func_id.func_name,)
             warnings.warn_explicit(warn_msg, errors.NumbaWarning,
-                                   self.func_attr.filename,
-                                   self.func_attr.lineno)
+                                   self.func_id.filename,
+                                   self.func_id.firstlineno)
             if self.flags.release_gil:
                 warn_msg = "Code running in object mode won't allow parallel execution despite nogil=True."
                 warnings.warn_explicit(warn_msg, errors.NumbaWarning,
-                                       self.func_attr.filename,
-                                       self.func_attr.lineno)
+                                       self.func_id.filename,
+                                       self.func_id.firstlineno)
 
     def stage_nopython_backend(self):
         """
@@ -624,7 +583,7 @@ class Pipeline(object):
         signature = typing.signature(types.pyobject, *args)
         self.cr = compile_result(typing_context=self.typingctx,
                                  target_context=self.targetctx,
-                                 entry_point=self.func,
+                                 entry_point=self.func_id.func,
                                  typing_error=self.status.fail_reason,
                                  type_annotation="<Interpreter mode function>",
                                  signature=signature,
@@ -637,8 +596,6 @@ class Pipeline(object):
         """
         Cleanup intermediate results to release resources.
         """
-        if self.interp is not None:
-            self.interp.reset()
 
     def _compile_core(self):
         """
@@ -648,8 +605,9 @@ class Pipeline(object):
 
         if not self.flags.force_pyobject:
             pm.create_pipeline("nopython")
-            if self.interp is None:
+            if self.func_ir is None:
                 pm.add_stage(self.stage_analyze_bytecode, "analyzing bytecode")
+            pm.add_stage(self.stage_process_ir, "processing IR")
             if not self.flags.no_rewrites:
                 pm.add_stage(self.stage_generic_rewrites, "nopython rewrites")
             pm.add_stage(self.stage_nopython_frontend, "nopython frontend")
@@ -661,8 +619,9 @@ class Pipeline(object):
 
         if self.status.can_fallback or self.flags.force_pyobject:
             pm.create_pipeline("object")
-            if self.interp is None:
+            if self.func_ir is None:
                 pm.add_stage(self.stage_analyze_bytecode, "analyzing bytecode")
+            pm.add_stage(self.stage_process_ir, "processing IR")
             pm.add_stage(self.stage_objectmode_frontend, "object mode frontend")
             pm.add_stage(self.stage_annotate_type, "annotate type")
             pm.add_stage(self.stage_objectmode_backend, "object mode backend")
@@ -686,14 +645,14 @@ class Pipeline(object):
         """
         Populate and run pipeline for bytecode input
         """
-        assert self.interp is None
+        assert self.func_ir is None
         return self._compile_core()
 
     def _compile_ir(self):
         """
         Populate and run pipeline for IR input
         """
-        assert self.interp is not None
+        assert self.func_ir is not None
         return self._compile_core()
 
 
@@ -725,22 +684,18 @@ def compile_extra(typingctx, targetctx, func, args, return_type, flags,
     return pipeline.compile_extra(func)
 
 
-def compile_bytecode(typingctx, targetctx, bc, args, return_type, flags,
-                     locals, lifted=(), lifted_from=None,
-                     func_attr=DEFAULT_FUNCTION_ATTRIBUTES, library=None):
+def compile_ir(typingctx, targetctx, func_ir, args, return_type, flags,
+               locals, lifted=(), lifted_from=None, library=None):
+    """
+    Compile a function with the given IR.
+
+    For internal use only.
+    """
 
     pipeline = Pipeline(typingctx, targetctx, library,
                         args, return_type, flags, locals)
-    return pipeline.compile_bytecode(bc=bc, lifted=lifted, lifted_from=lifted_from, func_attr=func_attr)
-
-
-def compile_ir(typingctx, targetctx, interp, args, return_type, flags,
-               locals, lifted=(), lifted_from=None,
-               func_attr=DEFAULT_FUNCTION_ATTRIBUTES, library=None):
-
-    pipeline = Pipeline(typingctx, targetctx, library,
-                        args, return_type, flags, locals)
-    return pipeline.compile_ir(interp=interp, lifted=lifted, lifted_from=lifted_from, func_attr=func_attr)
+    return pipeline.compile_ir(func_ir=func_ir, lifted=lifted,
+                               lifted_from=lifted_from)
 
 
 def compile_internal(typingctx, targetctx, library,
@@ -787,22 +742,24 @@ def legalize_return_type(return_type, interp, targetctx):
         raise TypeError("Can't return function object in nopython mode")
 
 
-def translate_stage(bytecode):
-    interp = interpreter.Interpreter(bytecode=bytecode)
+def translate_stage(func_id, bytecode):
+    interp = interpreter.Interpreter(func_id)
+    return interp.interpret(bytecode)
 
-    if config.DUMP_CFG:
-        interp.cfa.dump()
 
-    interp.interpret()
+def ir_processing_stage(func_ir):
+    post_proc = postproc.PostProcessor(func_ir)
+    post_proc.run()
 
     if config.DEBUG or config.DUMP_IR:
-        print(("IR DUMP: %s" % interp.bytecode.func_qualname).center(80, "-"))
-        interp.dump()
-        if interp.generator_info:
-            print(("GENERATOR INFO: %s" % interp.bytecode.func_qualname).center(80, "-"))
-            interp.dump_generator_info()
+        name = func_ir.func_id.func_qualname
+        print(("IR DUMP: %s" % name).center(80, "-"))
+        func_ir.dump()
+        if func_ir.is_generator:
+            print(("GENERATOR INFO: %s" % name).center(80, "-"))
+            func_ir.dump_generator_info()
 
-    return interp
+    return func_ir
 
 
 def type_inference_stage(typingctx, interp, args, return_type, locals={}):
@@ -811,22 +768,22 @@ def type_inference_stage(typingctx, interp, args, return_type, locals={}):
 
     warnings = errors.WarningsFixer(errors.NumbaWarning)
     infer = typeinfer.TypeInferer(typingctx, interp, warnings)
+    with typingctx.callstack.register(infer, interp.func_id, args):
+        # Seed argument types
+        for index, (name, ty) in enumerate(zip(interp.arg_names, args)):
+            infer.seed_argument(name, index, ty)
 
-    # Seed argument types
-    for index, (name, ty) in enumerate(zip(interp.bytecode.arg_names, args)):
-        infer.seed_argument(name, index, ty)
+        # Seed return type
+        if return_type is not None:
+            infer.seed_return(return_type)
 
-    # Seed return type
-    if return_type is not None:
-        infer.seed_return(return_type)
+        # Seed local types
+        for k, v in locals.items():
+            infer.seed_type(k, v)
 
-    # Seed local types
-    for k, v in locals.items():
-        infer.seed_type(k, v)
-
-    infer.build_constraint()
-    infer.propagate()
-    typemap, restype, calltypes = infer.unify()
+        infer.build_constraint()
+        infer.propagate()
+        typemap, restype, calltypes = infer.unify()
 
     # Output all Numba warnings
     warnings.flush()
