@@ -282,8 +282,21 @@ def detect():
 
 @contextlib.contextmanager
 def defer_cleanup():
-    tserv = get_current_device().trashing
-    with tserv.defer_cleanup:
+    """
+    Temporarily disable memory deallocation.
+    Use this to prevent resource deallocation breaking asynchronous execution.
+
+    For example::
+
+        with defer_cleanup():
+            # all cleanup is deferred in here
+            do_speed_critical_code()
+        # cleanup can occur here
+
+    Note: this context manager can be nested.
+    """
+    deallocs = current_context().deallocations
+    with deallocs.disable():
         yield
 
 
