@@ -9,6 +9,7 @@ import struct
 import sys
 import uuid
 import weakref
+import threading
 
 import numba
 from numba import _dispatcher, compiler, utils, types, config, errors
@@ -168,7 +169,7 @@ class _DispatcherBase(_dispatcher.Dispatcher):
                                         has_stararg)
 
         self.doc = py_func.__doc__
-        self._compile_lock = utils.NonReentrantLock()
+        self._compile_lock = threading.RLock()
 
         utils.finalize(self, self._make_finalizer())
 
@@ -242,8 +243,8 @@ class _DispatcherBase(_dispatcher.Dispatcher):
         # Fold keyword arguments and resolve default values
         pysig, args = self._compiler.fold_argument_types(args, kws)
         kws = {}
-        # Ensure an overload is available, but avoid compiler re-entrance
-        if self._can_compile and not self.is_compiling:
+        # Ensure an overload is available
+        if self._can_compile:
             self.compile(tuple(args))
 
         # Create function type for typing
@@ -267,7 +268,7 @@ class _DispatcherBase(_dispatcher.Dispatcher):
         """
         Whether a specialization is currently being compiled.
         """
-        return self._compile_lock.is_owned()
+        return self._compile_lock._is_owned()
 
     def _compile_for_args(self, *args, **kws):
         """
@@ -566,9 +567,8 @@ class LiftedLoop(_DispatcherBase):
     """
     _fold_args = False
 
-    def __init__(self, interp, typingctx, targetctx, flags, locals):
-        self.bytecode = interp.bytecode
-        self.interp = interp
+    def __init__(self, func_ir, typingctx, targetctx, flags, locals):
+        self.func_ir = func_ir
         self.lifted_from = None
 
         self.typingctx = typingctx
@@ -576,15 +576,14 @@ class LiftedLoop(_DispatcherBase):
         self.flags = flags
         self.locals = locals
 
-        _DispatcherBase.__init__(self, self.bytecode.arg_count,
-                                 self.bytecode.func, self.bytecode.pysig)
+        _DispatcherBase.__init__(self, self.func_ir.arg_count,
+                                 self.func_ir.func_id.func,
+                                 self.func_ir.func_id.pysig)
 
     def get_source_location(self):
         """Return the starting line number of the loop.
         """
-        firstblock = self.interp.blocks[min(self.interp.blocks)]
-        inst = firstblock.body[0]
-        return inst.loc.line
+        return self.func_ir.loc.line
 
     def compile(self, sig):
         with self._compile_lock:
@@ -601,7 +600,7 @@ class LiftedLoop(_DispatcherBase):
             assert not flags.enable_looplift, "Enable looplift flags is on"
             cres = compiler.compile_ir(typingctx=self.typingctx,
                                        targetctx=self.targetctx,
-                                       interp=self.interp,
+                                       func_ir=self.func_ir,
                                        args=args, return_type=return_type,
                                        flags=flags, locals=self.locals,
                                        lifted=(),
