@@ -11,6 +11,7 @@ from . import (_dynfunc, cgutils, config, funcdesc, generators, ir, types,
                typing, utils)
 from .errors import LoweringError, new_error_context
 from .targets import imputils
+from .funcdesc import default_mangler
 
 
 class Environment(_dynfunc.Environment):
@@ -44,14 +45,14 @@ class BaseLower(object):
     # If true, then can't cache LLVM module accross process calls
     has_dynamic_globals = False
 
-    def __init__(self, context, library, fndesc, interp):
+    def __init__(self, context, library, fndesc, func_ir):
         self.context = context
         self.library = library
         self.fndesc = fndesc
-        self.blocks = utils.SortedMap(utils.iteritems(interp.blocks))
-        self.interp = interp
+        self.blocks = utils.SortedMap(utils.iteritems(func_ir.blocks))
+        self.func_ir = func_ir
         self.call_conv = context.call_conv
-        self.generator_info = self.interp.generator_info
+        self.generator_info = func_ir.generator_info
 
         # Initialize LLVM
         self.module = self.library.create_ir_module(self.fndesc.unique_name)
@@ -534,9 +535,10 @@ class Lower(BaseLower):
                     self.builder, signature.args[index], default)
 
             def stararg_handler(index, param, vars):
+                stararg_ty = signature.args[index]
+                assert isinstance(stararg_ty, types.BaseTuple), stararg_ty
                 values = [self._cast_var(var, sigty)
-                          for var, sigty in
-                          zip(vars, signature.args[index])]
+                          for var, sigty in zip(vars, stararg_ty)]
                 return cgutils.make_anonymous_struct(self.builder, values)
 
             argvals = typing.fold_arguments(pysig,
@@ -643,9 +645,17 @@ class Lower(BaseLower):
                                                          argvals, fnty.cconv)
 
         elif isinstance(fnty, types.RecursiveCall):
-            # Self-recursive call
-            impl = imputils.user_function(self.fndesc, ())
-            res = impl(self.context, self.builder, signature, argvals)
+            # Recursive call
+            qualprefix = fnty.overloads[signature.args]
+            mangler = self.context.mangler or default_mangler
+            mangled_name = mangler(qualprefix, signature.args)
+            # special case self recursion
+            if self.builder.function.name.startswith(mangled_name):
+                res = self.context.call_internal(self.builder, self.fndesc,
+                                                 signature, argvals)
+            else:
+                res = self.context.call_unresolved(self.builder, mangled_name,
+                                                   signature, argvals)
 
         else:
             # Normal function resolution
