@@ -10,12 +10,12 @@ from llvmlite import ir
 
 import numpy as np
 
-from numba import jit, types, cgutils
+from numba import types, cgutils
 
 from numba.targets.imputils import (lower_builtin, impl_ret_borrowed,
                                     impl_ret_new_ref, impl_ret_untracked)
 from numba.typing import signature
-from numba.extending import overload
+from numba.extending import overload, register_jitable
 from numba.numpy_support import version as numpy_version
 from numba import types
 from numba import numpy_support as np_support
@@ -176,6 +176,19 @@ class _LAPACK:
                          )
         return types.ExternalFunction("numba_ez_cgeev", sig)
 
+    @classmethod
+    def numba_ez_xxxevd(cls, dtype):
+        wtype = getattr(dtype, "underlying_float", dtype)
+        sig = types.intc(types.char,             # kind
+                         types.char,             # jobz
+                         types.char,             # uplo
+                         types.intp,             # n
+                         types.CPointer(dtype),  # a
+                         types.intp,             # lda
+                         types.CPointer(wtype),  # w
+                         )
+        return types.ExternalFunction("numba_ez_xxxevd", sig)
+
     @classmethod
     def numba_xxpotrf(cls, dtype):
         sig = types.intc(types.char,             # kind
@@ -698,7 +711,7 @@ fatal_error_sig = types.intc()
 fatal_error_func = types.ExternalFunction("numba_fatal_error", fatal_error_sig)
 
 
-@jit(nopython=True)
+@register_jitable
 def _check_finite_matrix(a):
     for v in np.nditer(a):
         if not np.isfinite(v.item()):
@@ -726,7 +739,7 @@ def _check_homogeneous_types(func_name, *types):
             raise TypingError(msg)
 
 
-@jit(nopython=True)
+@register_jitable
 def _inv_err_handler(r):
     if r != 0:
         if r < 0:
@@ -780,7 +793,7 @@ def inv_impl(a):
     return inv_impl
 
 
-@jit(nopython=True)
+@register_jitable
 def _handle_err_maybe_convergence_problem(r):
     if r != 0:
         if r < 0:
@@ -968,6 +981,238 @@ if numpy_version >= (1, 8):
         else:
             return real_eig_impl
 
+    @overload(np.linalg.eigvals)
+    def eigvals_impl(a):
+        ensure_lapack()
+
+        _check_linalg_matrix(a, "eigvals")
+
+        numba_ez_rgeev = _LAPACK().numba_ez_rgeev(a.dtype)
+        numba_ez_cgeev = _LAPACK().numba_ez_cgeev(a.dtype)
+
+        kind = ord(get_blas_kind(a.dtype, "eigvals"))
+
+        JOBVL = ord('N')
+        JOBVR = ord('N')
+
+        F_layout = a.layout == 'F'
+
+        def real_eigvals_impl(a):
+            """
+            eigvals() implementation for real arrays.
+            """
+            n = a.shape[-1]
+            if a.shape[-2] != n:
+                msg = "Last 2 dimensions of the array must be square."
+                raise np.linalg.LinAlgError(msg)
+
+            _check_finite_matrix(a)
+
+            if F_layout:
+                acpy = np.copy(a)
+            else:
+                acpy = np.asfortranarray(a)
+
+            ldvl = 1
+            ldvr = 1
+            wr = np.empty(n, dtype=a.dtype)
+            wi = np.empty(n, dtype=a.dtype)
+
+            # not referenced but need setting for MKL null check
+            vl = np.empty((1), dtype=a.dtype)
+            vr = np.empty((1), dtype=a.dtype)
+
+            r = numba_ez_rgeev(kind,
+                               JOBVL,
+                               JOBVR,
+                               n,
+                               acpy.ctypes,
+                               n,
+                               wr.ctypes,
+                               wi.ctypes,
+                               vl.ctypes,
+                               ldvl,
+                               vr.ctypes,
+                               ldvr)
+            _handle_err_maybe_convergence_problem(r)
+
+            # By design numba does not support dynamic return types, however,
+            # Numpy does. Numpy uses this ability in the case of returning
+            # eigenvalues/vectors of a real matrix. The return type of
+            # np.linalg.eigvals(), when operating on a matrix in real space
+            # depends on the values present in the matrix itself (recalling
+            # that eigenvalues are the roots of the characteristic polynomial
+            # of the system matrix, which will by construction depend on the
+            # values present in the system matrix). As numba cannot handle
+            # the case of a runtime decision based domain change relative to
+            # the input type, if it is required numba raises as below.
+            if np.any(wi):
+                raise ValueError(
+                    "eigvals() argument must not cause a domain change.")
+
+            # put these in to help with liveness analysis,
+            # `.ctypes` doesn't keep the vars alive
+            acpy.size
+            vl.size
+            vr.size
+            wr.size
+            wi.size
+            return wr
+
+        def cmplx_eigvals_impl(a):
+            """
+            eigvals() implementation for complex arrays.
+            """
+            n = a.shape[-1]
+            if a.shape[-2] != n:
+                msg = "Last 2 dimensions of the array must be square."
+                raise np.linalg.LinAlgError(msg)
+
+            _check_finite_matrix(a)
+
+            if F_layout:
+                acpy = np.copy(a)
+            else:
+                acpy = np.asfortranarray(a)
+
+            ldvl = 1
+            ldvr = 1
+            w = np.empty(n, dtype=a.dtype)
+            vl = np.empty((1), dtype=a.dtype)
+            vr = np.empty((1), dtype=a.dtype)
+
+            r = numba_ez_cgeev(kind,
+                               JOBVL,
+                               JOBVR,
+                               n,
+                               acpy.ctypes,
+                               n,
+                               w.ctypes,
+                               vl.ctypes,
+                               ldvl,
+                               vr.ctypes,
+                               ldvr)
+            _handle_err_maybe_convergence_problem(r)
+
+            # put these in to help with liveness analysis,
+            # `.ctypes` doesn't keep the vars alive
+            acpy.size
+            vl.size
+            vr.size
+            w.size
+            return w
+
+        if isinstance(a.dtype, types.scalars.Complex):
+            return cmplx_eigvals_impl
+        else:
+            return real_eigvals_impl
+
+    @overload(np.linalg.eigh)
+    def eigh_impl(a):
+        ensure_lapack()
+
+        _check_linalg_matrix(a, "eigh")
+
+        F_layout = a.layout == 'F'
+
+        # convert typing floats to numpy floats for use in the impl
+        w_type = getattr(a.dtype, "underlying_float", a.dtype)
+        w_dtype = np_support.as_dtype(w_type)
+
+        numba_ez_xxxevd = _LAPACK().numba_ez_xxxevd(a.dtype)
+
+        kind = ord(get_blas_kind(a.dtype, "eigh"))
+
+        JOBZ = ord('V')
+        UPLO = ord('L')
+
+        def eigh_impl(a):
+            n = a.shape[-1]
+
+            if a.shape[-2] != n:
+                msg = "Last 2 dimensions of the array must be square."
+                raise np.linalg.LinAlgError(msg)
+
+            _check_finite_matrix(a)
+
+            if F_layout:
+                acpy = np.copy(a)
+            else:
+                acpy = np.asfortranarray(a)
+
+            w = np.empty(n, dtype=w_dtype)
+
+            r = numba_ez_xxxevd(kind,  # kind
+                                JOBZ,  # jobz
+                                UPLO,  # uplo
+                                n,  # n
+                                acpy.ctypes,  # a
+                                n,  # lda
+                                w.ctypes  # w
+                                )
+            _handle_err_maybe_convergence_problem(r)
+
+            # help liveness analysis
+            acpy.size
+            w.size
+
+            return (w, acpy)
+
+        return eigh_impl
+
+    @overload(np.linalg.eigvalsh)
+    def eigvalsh_impl(a):
+        ensure_lapack()
+
+        _check_linalg_matrix(a, "eigvalsh")
+
+        F_layout = a.layout == 'F'
+
+        # convert typing floats to numpy floats for use in the impl
+        w_type = getattr(a.dtype, "underlying_float", a.dtype)
+        w_dtype = np_support.as_dtype(w_type)
+
+        numba_ez_xxxevd = _LAPACK().numba_ez_xxxevd(a.dtype)
+
+        kind = ord(get_blas_kind(a.dtype, "eigvalsh"))
+
+        JOBZ = ord('N')
+        UPLO = ord('L')
+
+        def eigvalsh_impl(a):
+            n = a.shape[-1]
+
+            if a.shape[-2] != n:
+                msg = "Last 2 dimensions of the array must be square."
+                raise np.linalg.LinAlgError(msg)
+
+            _check_finite_matrix(a)
+
+            if F_layout:
+                acpy = np.copy(a)
+            else:
+                acpy = np.asfortranarray(a)
+
+            w = np.empty(n, dtype=w_dtype)
+
+            r = numba_ez_xxxevd(kind,  # kind
+                                JOBZ,  # jobz
+                                UPLO,  # uplo
+                                n,  # n
+                                acpy.ctypes,  # a
+                                n,  # lda
+                                w.ctypes  # w
+                                )
+            _handle_err_maybe_convergence_problem(r)
+
+            # help liveness analysis
+            acpy.size
+            w.size
+
+            return w
+
+        return eigvalsh_impl
+
     @overload(np.linalg.svd)
     def svd_impl(a, full_matrices=1):
         ensure_lapack()
@@ -1131,6 +1376,7 @@ def _system_copy_in_b(bcpy, b, nrhs):
     """
     raise NotImplementedError
 
+
 @overload(_system_copy_in_b)
 def _system_copy_in_b_impl(bcpy, b, nrhs):
     if b.ndim == 1:
@@ -1142,12 +1388,14 @@ def _system_copy_in_b_impl(bcpy, b, nrhs):
             bcpy[:b.shape[-2], :nrhs] = b
         return twoD_impl
 
+
 def _system_compute_nrhs(b):
     """
     Compute the number of right hand sides in the system of equations
     """
     raise NotImplementedError
 
+
 @overload(_system_compute_nrhs)
 def _system_compute_nrhs_impl(b):
     if b.ndim == 1:
@@ -1159,12 +1407,14 @@ def _system_compute_nrhs_impl(b):
             return b.shape[-1]
         return twoD_impl
 
+
 def _system_check_dimensionally_valid(a, b):
     """
     Check that AX=B style system input is dimensionally valid.
     """
     raise NotImplementedError
 
+
 @overload(_system_check_dimensionally_valid)
 def _system_check_dimensionally_valid_impl(a, b):
     ndim = b.ndim
@@ -1192,7 +1442,8 @@ def _lstsq_residual(b, n, rhs):
     """
     raise NotImplementedError
 
-@overload (_lstsq_residual)
+
+@overload(_lstsq_residual)
 def _lstsq_residual_impl(b, n, rhs):
     ndim = b.ndim
     dtype = b.dtype
@@ -1236,6 +1487,7 @@ def _lstsq_solution(b, bcpy, n):
     """
     raise NotImplementedError
 
+
 @overload(_lstsq_solution)
 def _lstsq_solution_impl(b, bcpy, n):
     if b.ndim == 1:
@@ -1357,6 +1609,7 @@ def _solve_compute_return(b, bcpy):
     """
     raise NotImplementedError
 
+
 @overload(_solve_compute_return)
 def _solve_compute_return_impl(b, bcpy):
     if b.ndim == 1:
@@ -1610,7 +1863,7 @@ def _get_slogdet_diag_walker(a):
     such that the log(value) stays in the real domain.
     """
     if isinstance(a.dtype, types.Complex):
-        @jit(nopython=True)
+        @register_jitable
         def cmplx_diag_walker(n, a, sgn):
             # walk diagonal
             csgn = sgn + 0.j
@@ -1622,7 +1875,7 @@ def _get_slogdet_diag_walker(a):
             return (csgn, acc)
         return cmplx_diag_walker
     else:
-        @jit(nopython=True)
+        @register_jitable
         def real_diag_walker(n, a, sgn):
             # walk diagonal
             acc = 0.
@@ -1801,6 +2054,7 @@ def _oneD_norm_2(a):
     """
     raise NotImplementedError
 
+
 @overload(_oneD_norm_2)
 def _oneD_norm_2_impl(a):
     nb_ret_type = getattr(a.dtype, "underlying_float", a.dtype)
@@ -1936,16 +2190,16 @@ def _get_norm_impl(a, ord_flag):
             # Force `a` to be C-order, so that we can take a contiguous
             # 1D view.
             if a.layout == 'C':
-                @jit(nopython=True)
+                @register_jitable
                 def array_prepare(a):
                     return a
             elif a.layout == 'F':
-                @jit(nopython=True)
+                @register_jitable
                 def array_prepare(a):
                     # Legal since L2(a) == L2(a.T)
                     return a.T
             else:
-                @jit(nopython=True)
+                @register_jitable
                 def array_prepare(a):
                     return a.copy()
 
@@ -2093,7 +2347,7 @@ def cond_impl(a, p=None):
     return _get_cond_impl(a, p)
 
 
-@jit(nopython=True)
+@register_jitable
 def _get_rank_from_singular_values(sv, t):
     """
     Gets rank from singular values with cut-off at a given tolerance
@@ -2171,3 +2425,71 @@ def matrix_rank_impl(a, tol=None):
             assert 0  # unreachable
 
     return _get_matrix_rank_impl(a, tol)
+
+
+@overload(np.linalg.matrix_power)
+def matrix_power_impl(a, n):
+    """
+    Computes matrix power. Only integer powers are supported in numpy.
+    """
+
+    _check_linalg_matrix(a, "matrix_power")
+    np_dtype = np_support.as_dtype(a.dtype)
+
+    nt = getattr(n, 'dtype', n)
+    if not isinstance(nt, types.Integer):
+        raise TypeError("Exponent must be an integer.")
+
+    def matrix_power_impl(a, n):
+
+        if n == 0:
+            # this should be eye() but it doesn't support
+            # the dtype kwarg yet so do it manually to save
+            # the copy required by eye(a.shape[0]).asdtype()
+            A = np.zeros(a.shape, dtype=np_dtype)
+            for k in range(a.shape[0]):
+                A[k, k] = 1.
+            return A
+
+        # note: to be consistent over contiguousness, C order is
+        # returned as that is what dot() produces and the most common
+        # paths through matrix_power will involve that. Therefore
+        # copies are made here to ensure the data ordering is
+        # correct for paths not going via dot().
+
+        if n < 0:
+            A = np.linalg.inv(a).copy()
+            if n == -1:  # return now
+                return A
+            n = -n
+        else:
+            if n == 1:  # return a copy now
+                return a.copy()
+            A = a  # this is safe, `a` is only read
+
+        if n < 4:
+            if n == 2:
+                return np.dot(A, A)
+            if n == 3:
+                return np.dot(np.dot(A, A), A)
+        else:
+
+            acc = A
+            exp = n
+
+            # tried a loop split and branchless using identity matrix as
+            # input but it seems like having a "first entry" flag is quicker
+            flag = True
+            while exp != 0:
+                if exp & 1:
+                    if flag:
+                        ret = acc
+                        flag = False
+                    else:
+                        ret = np.dot(ret, acc)
+                acc = np.dot(acc, acc)
+                exp = exp >> 1
+
+            return ret
+
+    return matrix_power_impl
