@@ -16,7 +16,6 @@ from numba import jit, vectorize
 from numba.config import PYVERSION
 from numba.errors import LoweringError, TypingError
 from .support import TestCase, CompilationCache, MemoryLeakMixin, tag
-from .test_operators import _avoid_windows_floordiv_heisenbug
 
 from numba.typing.npydecl import supported_ufuncs, all_ufuncs
 
@@ -336,9 +335,8 @@ class TestUFuncs(BaseUFuncTest, TestCase):
         self.binary_ufunc_test(np.true_divide, flags=flags, int_output_type=types.float64)
 
     @tag('important')
-    @_avoid_windows_floordiv_heisenbug
-    def test_floor_divide_ufunc(self, flags=no_pyobj_flags):
-        self.binary_ufunc_test(np.floor_divide, flags=flags)
+    def test_floor_divide_ufunc(self):
+        self.binary_ufunc_test(np.floor_divide)
 
     @tag('important')
     def test_negative_ufunc(self, flags=no_pyobj_flags):
@@ -353,7 +351,6 @@ class TestUFuncs(BaseUFuncTest, TestCase):
         self.binary_ufunc_test(np.power, flags=flags)
 
     @tag('important')
-    @_avoid_windows_floordiv_heisenbug
     def test_remainder_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.remainder, flags=flags)
 
@@ -1087,7 +1084,6 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
     def test_inplace_div(self):
         self.inplace_float_op_test('/=', [-1, 1.5, 3], [-5, 0, 2.5])
 
-    @_avoid_windows_floordiv_heisenbug
     def test_inplace_remainder(self):
         self.inplace_float_op_test('%=', [-1, 1.5, 3], [-5, 2, 2.5])
 
@@ -1156,7 +1152,6 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
         self.binary_op_test('/', int_output_type=int_out_type)
 
     @tag('important')
-    @_avoid_windows_floordiv_heisenbug
     def test_floor_divide_array_op(self):
         # Avoid floating-point zeros as x // 0.0 can have varying results
         # depending on the algorithm (which changed accross Numpy versions)
@@ -1184,7 +1179,6 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
         self.binary_op_test('//')
 
     @tag('important')
-    @_avoid_windows_floordiv_heisenbug
     def test_remainder_array_op(self):
         self.binary_op_test('%')
 
